<?php
namespace Http\Message\RequestMatcher;
use Http\Message\RequestMatcher as RequestMatcherInterface;
use Psr\Http\Message\RequestInterface;
/**
 * A port of the Symfony RequestMatcher for PSR-7.
 *
 * @author Fabien Potencier <fabien@symfony.com>
 * @author Joel Wurtz <joel.wurtz@gmail.com>
 */

final class RequestMatcher implements RequestMatcherInterface
{
    /**
     * @var string
     */
    private $path;
    /**
     * @var string
     */
    private $host;
    private $methods = [];
    private $schemes = [];
    public function __construct($path = null, $host = null, $methods = [], $schemes = [])
    {
        $this->path = $path;
        $this->host = $host;
        $this->methods = array_map('strtoupper', (array) $methods);
        $this->schemes = array_map('strtolower', (array) $schemes);
    }
    public function matches(RequestInterface $request)
    {
        if ($this->schemes && !in_array($request->getUri()->getScheme(), $this->schemes)) {
            return false;
        }
        if ($this->methods && !in_array($request->getMethod(), $this->methods)) {
            return false;
        }
        if (null !== $this->path && !preg_match('{'.$this->path.'}', rawurldecode($request->getUri()->getPath()))) {
            return false;
        }
        if (null !== $this->host && !preg_match('{'.$this->host.'}i', $request->getUri()->getHost())) {
            return false;
        }
        return true;
    }
}
