<?php
namespace PrestaShop\PsAccountsInstaller\Installer;
use PrestaShop\PrestaShop\Adapter\SymfonyContainer;
use PrestaShop\PrestaShop\Core\Addon\Module\ModuleManagerBuilder;
class Installer
{
    const PS_ACCOUNTS_MODULE_NAME = 'ps_accounts';
    /**
     * @var string required version
     */

    private $moduleVersion;
    /**
     * @var string
     */
    private $moduleName = self::PS_ACCOUNTS_MODULE_NAME;
    /**
     * @var \Link
     */
    private $link;
    private $moduleManager;
    public function __construct($psAccountsVersion, \Link $link = null)
    {
        $this->moduleVersion = $psAccountsVersion;
        if (null === $link) {
            $link = new \Link();
        }
        $this->link = $link;
        if (true === $this->isShopVersion17()) {
            $moduleManagerBuilder = ModuleManagerBuilder::getInstance();
            $this->moduleManager = $moduleManagerBuilder->build();
        }
    }
    public function install()
    {
        if (true === $this->isModuleInstalled()) {
            return true;
        }
        if (false === $this->isShopVersion17()) {
            return true;
        }
        return $this->moduleManager->install($this->getModuleName());
    }
    public function isModuleInstalled()
    {
        if (false === $this->isShopVersion17()) {
            return \Module::isInstalled($this->getModuleName());
        }
        return $this->moduleManager->isInstalled($this->getModuleName());
    }
    public function isModuleEnabled()
    {
        if (false === $this->isShopVersion17()) {
            return \Module::isEnabled($this->getModuleName());
        }
        return $this->moduleManager->isEnabled($this->getModuleName());
    }
    public function getInstallLink()
    {
        if ($this->isShopVersion173()) {
            $router = SymfonyContainer::getInstance()->get('router');
            return \Tools::getHttpHost(true) . $router->generate('admin_module_manage_action', [
                'action' => 'install',
                'module_name' => $this->moduleName,
            ]);
        }
        return $this->getAdminLink('AdminModules', true, [], [
            'module_name' => $this->moduleName,
            'install' => $this->moduleName,
        ]);
    }
    public function getEnableLink()
    {
        if ($this->isShopVersion173()) {
            $router = SymfonyContainer::getInstance()->get('router');
            return \Tools::getHttpHost(true) . $router->generate('admin_module_manage_action', [
                'action' => 'enable',
                'module_name' => $this->moduleName,
            ]);
        }
        return $this->getAdminLink('AdminModules', true, [], [
            'module_name' => $this->moduleName,
            'enable' => 1,
        ]);
    }
    public function getUpgradeLink()
    {
        if ($this->isShopVersion173()) {
            $router = SymfonyContainer::getInstance()->get('router');
            return \Tools::getHttpHost(true) . $router->generate('admin_module_manage_action', [
                'action' => 'upgrade',
                'module_name' => $this->moduleName,
            ]);
        }
        return $this->getAdminLink('AdminModules', true, [], [
            'module_name' => $this->moduleName,
            'upgrade' => $this->moduleName,
        ]);
    }
    public function isShopVersion17()
    {
        return version_compare(_PS_VERSION_, '1.7.0.0', '>=');
    }
    public function isShopVersion173()
    {
        return version_compare(_PS_VERSION_, '1.7.3.0', '>=');
    }
    public function checkModuleVersion()
    {
        $module = \Module::getInstanceByName($this->getModuleName());
        if ($module instanceof \Ps_accounts) {
            return version_compare(
                $module->version,
                $this->moduleVersion,
                '>='
            );
        }
        return false;
    }
    public function getModuleVersion()
    {
        return $this->moduleVersion;
    }
    public function getModuleName()
    {
        return $this->moduleName;
    }
    protected function getAdminLink($controller, $withToken = true, $sfRouteParams = [], $params = [])
    {
        if ($this->isShopVersion17()) {
            return $this->link->getAdminLink($controller, $withToken, $sfRouteParams, $params);
        }
        $paramsAsString = '';
        foreach ($params as $key => $value) {
            $paramsAsString .= "&$key=$value";
        }
        return \Tools::getShopDomainSsl(true)
            . __PS_BASE_URI__
            . basename(_PS_ADMIN_DIR_)
            . '/' . $this->link->getAdminLink($controller, $withToken)
            . $paramsAsString;
    }
}
