<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace PrestaShopCorp\Billing\Clients;
use Prestashop\ModuleLibGuzzleAdapter\ClientFactory;
class BillingClient extends GenericClient
{
    public const DEFAULT_API_VERSION = 'v1';
    public function __construct(
        $moduleName,
        $client = null,
        $apiUrl = null,
        $token = null,
        $isSandbox = null
    ) {
        parent::__construct();
        if (!isset($client) || null === $client) {
            $clientParams = [
                'base_url' => $apiUrl,
                'defaults' => [
                    'timeout' => $this->timeout,
                    'exceptions' => $this->catchExceptions,
                    'headers' => [
                        'Accept' => 'application/json',
                        'Authorization' => 'Bearer ' . (string) $token,
                        'User-Agent' => 'module-lib-billing (' . $moduleName . ')',
                    ],
                ],
            ];
            if (true === $isSandbox) {
                $clientParams['defaults']['headers']['Sandbox'] = 'true';
            }
            $client = (new ClientFactory())->getClient($clientParams);
        }
        $this->setClient($client);
        $this->setModuleName($moduleName);
    }
    public function retrieveCustomerById(string $customerId, string $apiVersion = self::DEFAULT_API_VERSION)
    {
        $this->setApiVersion($apiVersion);
        $this->setRoute('/customers/' . $customerId);
        return $this->get();
    }
    public function retrieveSubscriptionByCustomerId(string $customerId, string $apiVersion = self::DEFAULT_API_VERSION)
    {
        $this->setApiVersion($apiVersion);
        $this->setRoute('/customers/' . $customerId . '/subscriptions/' . $this->getModuleName());
        return $this->get();
    }
    public function retrievePlans(string $lang, string $apiVersion = self::DEFAULT_API_VERSION, string $status = 'active', int $limit = 10, string $offset = null)
    {
        $this->setApiVersion($apiVersion);
        $this->setRoute('/products/' . $this->getModuleName() . '/plans?status=' . $status . '&lang_iso_code=' . $lang . '&limit=' . $limit . ($offset ? '&offset=' . $offset : ''));
        return $this->get();
    }
    private $moduleName;
    private function getModuleName()
    {
        return $this->moduleName;
    }
    private function setModuleName(string $moduleName)
    {
        $this->moduleName = $moduleName;
    }
}
