<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace PrestaShopCorp\Billing\Clients;
use GuzzleHttp\Psr7\Request;
use PrestaShopCorp\Billing\Clients\Handler\HttpResponseHandler;
use Psr\Http\Client\ClientInterface;
abstract class GenericClient
{
    protected $catchExceptions = false;
    protected $client;
    protected $route;
    protected $timeout = 10;
    protected $apiVersion;
    public function __construct()
    {
    }
    protected function get(array $options = [])
    {
        $response = $this->getClient()->sendRequest(new Request('GET', $this->concatApiVersionAndRoute(), $options));
        $responseHandler = new HttpResponseHandler();
        return $responseHandler->handleResponse($response);
    }
    protected function setClient(ClientInterface $client)
    {
        $this->client = $client;
    }
    protected function setCatchExceptions(bool $bool)
    {
        $this->catchExceptions = $bool;
    }
    protected function setRoute(string $route)
    {
        $this->route = $route;
    }
    protected function setTimeout(int $timeout)
    {
        $this->timeout = $timeout;
    }
    protected function setApiVersion($apiVersion)
    {
        $this->apiVersion = $apiVersion;
    }
    protected function isCatchExceptions()
    {
        return $this->catchExceptions;
    }
    public function getClient()
    {
        return $this->client;
    }
    protected function getRoute()
    {
        return $this->route;
    }
    protected function getTimeout()
    {
        return $this->timeout;
    }
    protected function getApiVersion()
    {
        return $this->apiVersion;
    }
    private function concatApiVersionAndRoute()
    {
        if ($this->getApiVersion()) {
            return $this->getApiVersion() . $this->getRoute();
        }
        return $this->getRoute();
    }
}
