<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace PrestaShopCorp\Billing\Services;
use Module;
use PrestaShopCorp\Billing\Builder\UrlBuilder;
use PrestaShopCorp\Billing\Clients\BillingClient;
use PrestaShopCorp\Billing\Wrappers\BillingContextWrapper;
class BillingService
{
    private $billingClient;
    private $apiVersion;
    private $billingContextWrapper;
    public function __construct(
        BillingContextWrapper $billingContextWrapper = null,
        Module $module,
        $apiVersion = BillingClient::DEFAULT_API_VERSION,
        $apiUrl = null
    ) {
        $this->setBillingContextWrapper($billingContextWrapper);
        $urlBuilder = new UrlBuilder();
        $this->setBillingClient(new BillingClient(
            $module->name,
            null,
            $urlBuilder->buildAPIUrl($this->getBillingContextWrapper()->getBillingEnv(), $apiUrl),
            $this->getBillingContextWrapper()->getAccessToken(),
            $this->getBillingContextWrapper()->isSandbox()
        ));
        $this->setApiVersion($apiVersion);
    }
    public function getCurrentCustomer()
    {
        return $this->getBillingClient()->retrieveCustomerById($this->getBillingContextWrapper()->getShopUuid(), $this->getApiVersion());
    }
    public function getCurrentSubscription()
    {
        return $this->getBillingClient()->retrieveSubscriptionByCustomerId($this->getBillingContextWrapper()->getShopUuid(), $this->getApiVersion());
    }
    public function getModulePlans()
    {
        return $this->getBillingClient()->retrievePlans($this->getBillingContextWrapper()->getLanguageIsoCode(), $this->getApiVersion());
    }
    public function setApiVersion(string $apiVersion)
    {
        $this->apiVersion = $apiVersion;
    }
    private function getApiVersion()
    {
        return $this->apiVersion;
    }
    public function setBillingClient(BillingClient $billingClient)
    {
        $this->billingClient = $billingClient;
    }
    private function getBillingClient()
    {
        return $this->billingClient;
    }
    private function setBillingContextWrapper(BillingContextWrapper $billingContextWrapper)
    {
        $this->billingContextWrapper = $billingContextWrapper;
    }
    private function getBillingContextWrapper()
    {
        return $this->billingContextWrapper;
    }
}
