<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace PrestaShopCorp\Billing\Wrappers;
use PrestaShop\PsAccountsInstaller\Installer\Facade\PsAccounts;
use PrestaShopCorp\Billing\Config\Config;
class BillingContextWrapper
{
    private $psAccountsService;
    private $context;
    private $sandbox;
    private $billingEnv;
    public function __construct(
        PsAccounts $accountFacade = null,
        \Context $context = null,
        $sandbox = false,
        $billingEnv = null
    ) {
        if (null === $context) {
            $context = \Context::getContext();
        }
        $this->setContext($context);
        $this->psAccountsService = $accountFacade ? $accountFacade->getPsAccountsService() : \Module::getInstanceByName(Config::PS_ACCOUNTS_MODULE_NAME)->getService(Config::PS_ACCOUNTS_SERVICE);
        $this->setSandbox($sandbox);
        $this->setBillingEnv($billingEnv);
    }
    public function getLanguageIsoCode()
    {
        return $this->getContext()->language !== null ? $this->getContext()->language->iso_code : Config::I18N_FALLBACK_LOCALE;
    }
    public function getShopUuid()
    {
        return method_exists($this->getPsAccountService(), 'getShopUuid') ? $this->getPsAccountService()->getShopUuid() : $this->getPsAccountService()->getShopUuidV4();
    }
    public function getRefreshToken()
    {
        return $this->getPsAccountService()->getRefreshToken();
    }
    public function getAccessToken()
    {
        return $this->getPsAccountService()->getOrRefreshToken();
    }
    public function getEmail()
    {
        return $this->getPsAccountService()->getEmail();
    }
    public function isSandbox()
    {
        return $this->sandbox;
    }
    public function getBillingEnv()
    {
        return $this->billingEnv;
    }
    private function getPsAccountService()
    {
        return $this->psAccountsService;
    }
    private function setContext(\Context $context)
    {
        $this->context = $context;
    }
    private function getContext()
    {
        return $this->context;
    }
    private function setSandbox(bool $sandbox)
    {
        $this->sandbox = $sandbox;
    }
    private function setBillingEnv($billingEnv)
    {
        $this->billingEnv = $billingEnv;
    }
}
