<?php
declare(strict_types=1);
namespace Psr\Http\Message;
/**
 * HTTP messages consist of requests from a client to a server and responses
 * from a server to a client. This interface defines the methods common to
 * each.
 *
 * Messages are considered immutable; all methods that might change state MUST
 * be implemented such that they retain the internal state of the current
 * message and return an instance that contains the changed state.
 *
 * @link http://www.ietf.org/rfc/rfc7230.txt
 * @link http://www.ietf.org/rfc/rfc7231.txt
 */

interface MessageInterface
{
    /**
     * Retrieves the HTTP protocol version as a string.
     *
     * The string MUST contain only the HTTP version number (e.g., "1.1", "1.0").
     *
     * @return string HTTP protocol version.
     */
    public function getProtocolVersion();
    public function withProtocolVersion(string $version);
    public function getHeaders();
    public function hasHeader(string $name);
    public function getHeader(string $name);
    public function getHeaderLine(string $name);
    public function withHeader(string $name, $value);
    public function withAddedHeader(string $name, $value);
    public function withoutHeader(string $name);
    public function getBody();
    public function withBody(StreamInterface $body);
}
